# -*- coding: utf-8 -*-
## Product description
## 
## Copyright (C) 2006 Ingeniweb

## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.

## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.

## You should have received a copy of the GNU General Public License
## along with this program; see the file COPYING. If not, write to the
## Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
"""
Multi-page content type stuff
"""

# Zope imports
from AccessControl import ClassSecurityInfo

# CMF imports
from Products.CMFCore import permissions as CCP

# Archetypes imports
try:
    from Products.LinguaPlone.public import registerType, Schema
except ImportError:
    # No multilingual support
    from Products.Archetypes.public import registerType, Schema

from Products.Archetypes.Field import BooleanField
from Products.Archetypes.Widget import BooleanWidget

from Products.ATContentTypes.content.base import ATCTOrderedFolder
from Products.ATContentTypes.lib.constraintypes import ConstrainTypesMixinSchema

from Products.PloneArticle.config import PLONEARTICLE_TOOL, PLONE_2_5
from Products.PloneArticle.interfaces import IPloneArticleMultiPage

schema = ATCTOrderedFolder.schema.copy() + ConstrainTypesMixinSchema.copy() \
         + Schema((
    BooleanField(
        'viewTOCFirst',
        default=False,
        widget=BooleanWidget(
            label='View table of contents first',
            label_msgid='label_view_toc_first',
            i18n_domain='plonearticle',
            visible={'view' : 'invisible'},
        )
    ),
    BooleanField(
        'viewTOC',
        default=True,
        widget=BooleanWidget(
            label='Display table of contents in dropdown list',
            label_msgid='label_display_toc_dropdown',
            description='If you have checked properties to view TOC first, this property will not be applied.',
            description_msgid='description_display_toc_dropdown',
            i18n_domain='plonearticle',
            visible={'view' : 'invisible'},
        )
    ),
    ))

class PloneArticleMultiPage(ATCTOrderedFolder):
    """
    """

    portal_type = meta_type = 'PloneArticleMultiPage'
    archetype_name = 'Multipage article'
    content_icon   = 'multipagearticle_icon.gif'
    typeDescription = 'A document holding a set of articles'
    typeDescMsgId = 'description_edit_multipage'
    #immediate_view = 'plonearticlemultipage_view'
    #default_view   = 'plonearticlemultipage_view'
    #suppl_views    = ()
    global_allow = True
    filter_content_types = True
    allowed_content_types = ('PloneArticle',)
    schema = schema

    security = ClassSecurityInfo()
    __implements__ = ATCTOrderedFolder.__implements__ + (
        IPloneArticleMultiPage,
        )

    # Make sure we get title-to-id generation when an object is created
    _at_rename_after_creation = True

    security.declareProtected(CCP.View, 'getPages')
    def getPages(self, full_objects=False):
        """
        Return the ordered list of articles as brains
        """
        return self.getFolderContents(full_objects)

    security.declareProtected(CCP.View, 'getViewTOC')
    def getViewTOC(self):
        """Returns true if you can access TOC in dropdown menu"""
        
        field = self.getField('viewTOC')
        value = field.get(self)
        
        if not value and not self.getViewTOCFirst():
            return False
        
        return True


    security.declarePublic('canSetDefaultPage')
    def canSetDefaultPage(self):
        """
        Always False: the default page is automatic for multipage
        """
        return False
    
    security.declareProtected(CCP.View, 'getDefaultPage')
    def getDefaultPage(self):
        """
        return the first user visible article as default page if
        ViewTOC is unchecked (i.e, it redirect the user to the first page)
        """
        request = getattr(self, 'REQUEST', None)
        force_toc = request.get('force_toc', False)
        ids = self.objectIds() 
        noPage = len(ids) == 0
   
        if force_toc or self.getViewTOCFirst() or noPage:
            return None
        else:            
            return ids[0]

    # Plone 2.1: monkey patch applied, we must do that to have a dynamic page
    # For Plone 2.5 we have a 'default_page' five view.
    if not PLONE_2_5:
        default_page = getDefaultPage
        
registerType(PloneArticleMultiPage)
